@@ -47,18 +47,33 @@ void HeaderGenerator::writeCopyCtor(QTextStream& s, const AbstractMetaClass* met
 
 void HeaderGenerator::writeProtectedFieldAccessors(QTextStream& s, const AbstractMetaField* field) const
 {
-    QString fieldType = field->type()->cppSignature();
+    AbstractMetaType *metaType = field->type();
+    QString fieldType = metaType->cppSignature();
     QString fieldName = field->enclosingClass()->qualifiedCppName() + "::" + field->name();
 
-    s << INDENT << "inline " << fieldType << ' ' << protectedFieldGetterName(field) << "()";
-    s << " { return " << fieldName << "; }" << endl;
-    s << INDENT << "inline void " << protectedFieldSetterName(field) << '(' << fieldType << " value)";
-    s << " { " << fieldName << " = value; }" << endl;
+    // Force use of pointer to return internal variable memory
+    bool useReference = (!metaType->isConstant() &&
+                         !metaType->isEnum() &&
+                         !metaType->isPrimitive() &&
+                         metaType->indirections() == 0);
+
+
+    // Get function
+    s << INDENT << "inline " << fieldType
+      << (useReference ? '*' : ' ')
+      << ' ' << protectedFieldGetterName(field) << "()"
+      << " { return "
+      << (useReference ? '&' : ' ') << "this->" << fieldName << "; }" << endl;
+
+    // Set function
+    s << INDENT << "inline void " << protectedFieldSetterName(field) << '(' << fieldType << " value)"
+      << " { " << fieldName << " = value; }" << endl;
 }
 
 void HeaderGenerator::generateClass(QTextStream& s, const AbstractMetaClass* metaClass)
 {
     ReportHandler::debugSparse("Generating header for " + metaClass->fullName());
+    m_inheritedOverloads.clear();
     Indentation indent(INDENT);
 
     // write license comment
@@ -71,9 +86,8 @@ void HeaderGenerator::generateClass(QTextStream& s, const AbstractMetaClass* met
     s << "#ifndef SBK_" << headerGuard << "_H" << endl;
     s << "#define SBK_" << headerGuard << "_H" << endl<< endl;
 
-#ifndef AVOID_PROTECTED_HACK
-    s << "#define protected public" << endl << endl;
-#endif
+    if (!avoidProtectedHack())
+        s << "#define protected public" << endl << endl;
 
     s << "#include <shiboken.h>" << endl << endl;
 
@@ -91,9 +105,6 @@ void HeaderGenerator::generateClass(QTextStream& s, const AbstractMetaClass* met
 
         s << endl << '{' << endl << "public:" << endl;
 
-        if (metaClass->typeEntry()->isValue())
-            writeCopyCtor(s, metaClass);
-
         bool hasVirtualFunction = false;
         foreach (AbstractMetaFunction *func, filterFunctions(metaClass)) {
             if (func->isVirtual())
@@ -101,38 +112,38 @@ void HeaderGenerator::generateClass(QTextStream& s, const AbstractMetaClass* met
             writeFunction(s, func);
         }
 
-#ifdef AVOID_PROTECTED_HACK
-        if (metaClass->hasProtectedFields()) {
+        if (avoidProtectedHack() && metaClass->hasProtectedFields()) {
             foreach (AbstractMetaField* field, metaClass->fields()) {
                 if (!field->isProtected())
                     continue;
                 writeProtectedFieldAccessors(s, field);
             }
         }
-#endif
 
         //destructor
-#ifdef AVOID_PROTECTED_HACK
-        if (!metaClass->hasPrivateDestructor())
-#endif
-            s << INDENT << (metaClass->hasVirtualDestructor() || hasVirtualFunction ? "virtual " : "") << "~" << wrapperName << "();" << endl;
+        if (!avoidProtectedHack() || !metaClass->hasPrivateDestructor()) {
+            s << INDENT;
+            if (metaClass->hasVirtualDestructor() || hasVirtualFunction)
+                s <<  "virtual ";
+            s << "~" << wrapperName << "();" << endl;
+        }
 
         writeCodeSnips(s, metaClass->typeEntry()->codeSnips(), CodeSnip::Declaration, TypeSystem::NativeCode);
 
-#ifdef AVOID_PROTECTED_HACK
-        if (!metaClass->hasPrivateDestructor()) {
-#endif
-
-        if (usePySideExtensions() && metaClass->isQObject()) {
+        if ((!avoidProtectedHack() || !metaClass->hasPrivateDestructor())
+            && usePySideExtensions() && metaClass->isQObject()) {
             s << "public:\n";
-            s << INDENT << "virtual int qt_metacall(QMetaObject::Call call, int id, void** args);\n";
-            s << "private:\n";
-            s << INDENT << "mutable PySide::DynamicQMetaObject* m_metaObject;\n";
+            s << INDENT << "virtual int qt_metacall(QMetaObject::Call call, int id, void** args);" << endl;
+            s << INDENT << "virtual void* qt_metacast(const char* _clname);" << endl;
         }
 
-#ifdef AVOID_PROTECTED_HACK
+        if (m_inheritedOverloads.size()) {
+            s << INDENT << "// Inherited overloads, because the using keyword sux" << endl;
+            writeInheritedOverloads(s);
         }
-#endif
+
+        if (usePySideExtensions())
+            s << INDENT << "static void pysideInitQtMetaTypes();" << endl;
 
         s << "};" << endl << endl;
     }
@@ -140,17 +151,18 @@ void HeaderGenerator::generateClass(QTextStream& s, const AbstractMetaClass* met
     s << "#endif // SBK_" << headerGuard << "_H" << endl << endl;
 }
 
-void HeaderGenerator::writeFunction(QTextStream& s, const AbstractMetaFunction* func) const
+void HeaderGenerator::writeFunction(QTextStream& s, const AbstractMetaFunction* func)
 {
+
     // do not write copy ctors here.
-    if (func->isCopyConstructor())
+    if (!func->isPrivate() && func->isCopyConstructor()) {
+        writeCopyCtor(s, func->ownerClass());
+        return;
+    }
+    if (func->isUserAdded())
         return;
 
-    if (func->isConstructor() && func->isUserAdded())
-        return;
-
-#ifdef AVOID_PROTECTED_HACK
-    if (func->isProtected() && !func->isConstructor() && !func->isOperatorOverload()) {
+    if (avoidProtectedHack() && func->isProtected() && !func->isConstructor() && !func->isOperatorOverload()) {
         s << INDENT << "inline " << (func->isStatic() ? "static " : "");
         s << functionSignature(func, "", "_protected", Generator::EnumAsInts|Generator::OriginalTypeDescription) << " { ";
         s << (func->type() ? "return " : "");
@@ -172,17 +184,15 @@ void HeaderGenerator::writeFunction(QTextStream& s, const AbstractMetaFunction*
         s << args.join(", ") << ')';
         s << "; }" << endl;
     }
-#endif
 
     // pure virtual functions need a default implementation
     if ((func->isPrivate() && !visibilityModifiedToPrivate(func))
         || (func->isModifiedRemoved() && !func->isAbstract()))
         return;
 
-#ifdef AVOID_PROTECTED_HACK
-    if (func->ownerClass()->hasPrivateDestructor() && (func->isAbstract() || func->isVirtual()))
+    if (avoidProtectedHack() && func->ownerClass()->hasPrivateDestructor()
+        && (func->isAbstract() || func->isVirtual()))
         return;
-#endif
 
     if (func->isConstructor() || func->isAbstract() || func->isVirtual()) {
         s << INDENT;
@@ -195,6 +205,20 @@ void HeaderGenerator::writeFunction(QTextStream& s, const AbstractMetaFunction*
 
         s << functionSignature(func, "", "", virtualOption) << ';' << endl;
 
+        // Check if this method hide other methods in base classes
+        foreach (const AbstractMetaFunction* f, func->ownerClass()->functions()) {
+            if (f != func
+                && !f->isConstructor()
+                && !f->isPrivate()
+                && !f->isVirtual()
+                && !f->isAbstract()
+                && !f->isStatic()
+                && f->name() == func->name()) {
+                m_inheritedOverloads << f;
+                break;
+            }
+        }
+
         // TODO: when modified an abstract method ceases to be virtual but stays abstract
         //if (func->isModifiedRemoved() && func->isAbstract()) {
         //}
@@ -215,18 +239,21 @@ void HeaderGenerator::writeTypeConverterDecl(QTextStream& s, const TypeEntry* ty
     }
     bool isValueTypeWithImplConversions = type->isValue() && !implicitConvs.isEmpty();
     bool hasCustomConversion = type->hasNativeConversionRule();
-    QString typeT = type->name() + (isAbstractOrObjectType ? "*" : "");
-    QString typeName = type->name();
+    QString typeT = type->qualifiedCppName() + (isAbstractOrObjectType ? "*" : "");
+    QString typeName = type->qualifiedCppName();
 
-#ifdef AVOID_PROTECTED_HACK
-    const AbstractMetaEnum* metaEnum = findAbstractMetaEnum(type);
-    if (metaEnum && metaEnum->isProtected()) {
-        typeT = protectedEnumSurrogateName(metaEnum);
-        typeName = typeT;
+    if (avoidProtectedHack()) {
+        const AbstractMetaEnum* metaEnum = findAbstractMetaEnum(type);
+        if (metaEnum && metaEnum->isProtected()) {
+            typeT = protectedEnumSurrogateName(metaEnum);
+            typeName = typeT;
+        }
     }
-#endif
 
-    s << "struct Converter<" << typeT << " >";
+    typeT.prepend("::");
+    typeName.prepend("::");
+
+    s << "struct Converter< " << typeT << " >";
     if (!hasCustomConversion) {
         if (type->isEnum())
             s << " : EnumConverter";
@@ -236,17 +263,17 @@ void HeaderGenerator::writeTypeConverterDecl(QTextStream& s, const TypeEntry* ty
             s << " : ObjectTypeConverter";
         else
             s << " : ValueTypeConverter";
-        s << '<' << typeName << " >";
+        s << "< " << typeName << " >";
     }
     s << endl << '{' << endl;
     if (isValueTypeWithImplConversions || hasCustomConversion) {
-        s << INDENT << "static " << type->name() << " toCpp(PyObject* pyobj);" << endl;
+        s << INDENT << "static " << typeName << " toCpp(PyObject* pyobj);" << endl;
         s << INDENT << "static bool isConvertible(PyObject* pyobj);" << endl;
         if (hasCustomConversion) {
             s << INDENT << "static bool checkType(PyObject* pyobj);" << endl;
-            s << INDENT << "static inline PyObject* toPython(void* cppObj) { return toPython(*reinterpret_cast<"
-              << type->name() << (isAbstractOrObjectType ? "" : "*") << " >(cppObj)); }" << endl;
-            s << INDENT << "static PyObject* toPython(const " << type->name() << "& cppObj);" << endl;
+            s << INDENT << "static inline PyObject* toPython(void* cppObj) { return toPython(*reinterpret_cast< "
+              << typeName << (isAbstractOrObjectType ? "" : "*") << " >(cppObj)); }" << endl;
+            s << INDENT << "static PyObject* toPython(const " << typeName << "& cppObj);" << endl;
         }
     }
     s << "};" << endl;
@@ -254,12 +281,12 @@ void HeaderGenerator::writeTypeConverterDecl(QTextStream& s, const TypeEntry* ty
     // write value-type like converter to object-types
     if (isAbstractOrObjectType) {
         s << endl << "template<>" << endl;
-        s << "struct Converter<" << type->name() << "& > : ObjectTypeReferenceConverter<" << type->name() << " >" << endl << '{' << endl;
+        s << "struct Converter< " << typeName << " > : ObjectTypeReferenceConverter< " << typeName << " >" << endl << '{' << endl;
         s << "};" << endl;
     }
 }
 
-void HeaderGenerator::writeTypeIndexDefineLine(QTextStream& s, const TypeEntry* typeEntry, int& idx)
+void HeaderGenerator::writeTypeIndexDefineLine(QTextStream& s, const TypeEntry* typeEntry)
 {
     if (!typeEntry || !typeEntry->generateCode())
         return;
@@ -268,23 +295,23 @@ void HeaderGenerator::writeTypeIndexDefineLine(QTextStream& s, const TypeEntry*
     s.setFieldWidth(60);
     s << getTypeIndexVariableName(typeEntry);
     s.setFieldWidth(0);
-    s << ' ' << (idx++) << endl;
+    s << ' ' << getTypeIndex(typeEntry) << endl;
     if (typeEntry->isEnum()) {
         const EnumTypeEntry* ete = reinterpret_cast<const EnumTypeEntry*>(typeEntry);
         if (ete->flags())
-            writeTypeIndexDefineLine(s, ete->flags(), idx);
+            writeTypeIndexDefineLine(s, ete->flags());
     }
 }
 
-void HeaderGenerator::writeTypeIndexDefine(QTextStream& s, const AbstractMetaClass* metaClass, int& idx)
+void HeaderGenerator::writeTypeIndexDefine(QTextStream& s, const AbstractMetaClass* metaClass)
 {
     if (!metaClass->typeEntry()->generateCode())
         return;
-    writeTypeIndexDefineLine(s, metaClass->typeEntry(), idx);
+    writeTypeIndexDefineLine(s, metaClass->typeEntry());
     foreach (const AbstractMetaEnum* metaEnum, metaClass->enums()) {
         if (metaEnum->isPrivate())
             continue;
-        writeTypeIndexDefineLine(s, metaEnum->typeEntry(), idx);
+        writeTypeIndexDefineLine(s, metaEnum->typeEntry());
     }
 }
 
@@ -305,30 +332,35 @@ void HeaderGenerator::finishGeneration()
     QTextStream typeFunctions(&sbkTypeFunctions);
     QString converterImpl;
     QTextStream convImpl(&converterImpl);
+    QString protectedEnumSurrogates;
+    QTextStream protEnumsSurrogates(&protectedEnumSurrogates);
 
     Indentation indent(INDENT);
 
     macrosStream << "// Type indices" << endl;
-    int idx = 0;
-    foreach (const AbstractMetaClass* metaClass, classes())
-        writeTypeIndexDefine(macrosStream, metaClass, idx);
-    foreach (const AbstractMetaEnum* metaEnum, globalEnums())
-        writeTypeIndexDefineLine(macrosStream, metaEnum->typeEntry(), idx);
+    AbstractMetaEnumList globalEnums = this->globalEnums();
+    foreach (const AbstractMetaClass* metaClass, classes()) {
+        writeTypeIndexDefine(macrosStream, metaClass);
+        lookForEnumsInClassesNotToBeGenerated(globalEnums, metaClass);
+    }
+    foreach (const AbstractMetaEnum* metaEnum, globalEnums)
+        writeTypeIndexDefineLine(macrosStream, metaEnum->typeEntry());
     macrosStream << "#define ";
     macrosStream.setFieldWidth(60);
     macrosStream << "SBK_"+moduleName()+"_IDX_COUNT";
     macrosStream.setFieldWidth(0);
-    macrosStream << ' ' << idx << endl << endl;
+    macrosStream << ' ' << getMaxTypeIndex() << endl << endl;
     macrosStream << "// This variable stores all python types exported by this module" << endl;
     macrosStream << "extern PyTypeObject** " << cppApiVariableName() << ';' << endl << endl;
 
     macrosStream << "// Macros for type check" << endl;
-    foreach (const AbstractMetaEnum* cppEnum, globalEnums()) {
+    foreach (const AbstractMetaEnum* cppEnum, globalEnums) {
         if (cppEnum->isAnonymous() || cppEnum->isPrivate())
             continue;
         includes << cppEnum->typeEntry()->include();
         writeTypeConverterDecl(convDecl, cppEnum->typeEntry());
         convDecl << endl;
+        writeProtectedEnumSurrogate(protEnumsSurrogates, cppEnum);
         writeSbkTypeFunction(typeFunctions, cppEnum);
     }
 
@@ -350,12 +382,12 @@ void HeaderGenerator::finishGeneration()
             if (flagsEntry)
                 writeTypeConverterDecl(convDecl, flagsEntry);
             convDecl << endl;
+            writeProtectedEnumSurrogate(protEnumsSurrogates, cppEnum);
             writeSbkTypeFunction(typeFunctions, cppEnum);
         }
 
         if (!metaClass->isNamespace()) {
             writeSbkTypeFunction(typeFunctions, metaClass);
-            writeSbkCopyCppObjectFunction(convDecl, metaClass);
             writeTypeConverterDecl(convDecl, classType);
             writeTypeConverterImpl(convImpl, classType);
             convDecl << endl;
@@ -375,19 +407,19 @@ void HeaderGenerator::finishGeneration()
 
     s << "#ifndef " << includeShield << endl;
     s << "#define " << includeShield << endl<< endl;
-    #ifndef AVOID_PROTECTED_HACK
-    s << "//workaround to access protected functions" << endl;
-    s << "#define protected public" << endl << endl;
-    #endif
+    if (!avoidProtectedHack()) {
+        s << "//workaround to access protected functions" << endl;
+        s << "#define protected public" << endl << endl;
+    }
 
     s << "#include <Python.h>" << endl;
     s << "#include <conversions.h>" << endl;
-    s << "#include <pyenum.h>" << endl;
+    s << "#include <sbkenum.h>" << endl;
     s << "#include <basewrapper.h>" << endl;
     s << "#include <bindingmanager.h>" << endl;
     s << "#include <memory>" << endl << endl;
     if (usePySideExtensions())
-        s << "#include <qsignal.h>" << endl;
+        s << "#include <pysidesignal.h>" << endl;
 
     QStringList requiredTargetImports = TypeDatabase::instance()->requiredTargetImports();
     if (!requiredTargetImports.isEmpty()) {
@@ -417,6 +449,11 @@ void HeaderGenerator::finishGeneration()
 
     s << macros << endl;
 
+    if (!protectedEnumSurrogates.isEmpty()) {
+        s << "// Protected enum surrogates" << endl;
+        s << protectedEnumSurrogates << endl;
+    }
+
     s << "namespace Shiboken" << endl << '{' << endl << endl;
 
     s << "// PyType functions, to get the PyObjectType for a type T\n";
@@ -435,9 +472,9 @@ void HeaderGenerator::finishGeneration()
             s << INDENT << metaClass->qualifiedCppName() << "* value = const_cast<" << metaClass->qualifiedCppName() << "* >(cppobj);" << endl;
             s << INDENT << "if (!isExactType)" << endl;
             s << INDENT << INDENT << "typeName = typeid(*value).name();" << endl;
-            s << INDENT << "PyObject* pyObj = Shiboken::SbkBaseWrapper_New(reinterpret_cast<SbkBaseWrapperType*>(SbkType<" << metaClass->qualifiedCppName() << " >()),"
+            s << INDENT << "PyObject* pyObj = Shiboken::Object::newObject(reinterpret_cast<SbkObjectType*>(SbkType< ::" << metaClass->qualifiedCppName() << " >()),"
             << "value, hasOwnership, isExactType, typeName);" << endl;
-            s << INDENT << "PySide::signalUpdateSource(pyObj);" << endl;
+            s << INDENT << "PySide::Signal::updateSourceObject(pyObj);" << endl;
             s << INDENT << "return pyObj;" << endl;
             s << '}' << endl;
 
@@ -461,46 +498,39 @@ void HeaderGenerator::finishGeneration()
     s << "#endif // " << includeShield << endl << endl;
 }
 
+void HeaderGenerator::writeProtectedEnumSurrogate(QTextStream& s, const AbstractMetaEnum* cppEnum)
+{
+    if (avoidProtectedHack() && cppEnum->isProtected())
+        s << "enum " << protectedEnumSurrogateName(cppEnum) << " {};" << endl;
+}
+
 void HeaderGenerator::writeSbkTypeFunction(QTextStream& s, const AbstractMetaEnum* cppEnum)
 {
-    QString enumName = cppEnum->name();
-    if (cppEnum->enclosingClass())
-        enumName = cppEnum->enclosingClass()->qualifiedCppName() + "::" + enumName;
-#ifdef AVOID_PROTECTED_HACK
-    if (cppEnum->isProtected()) {
+    QString enumName;
+    if (avoidProtectedHack() && cppEnum->isProtected()) {
         enumName = protectedEnumSurrogateName(cppEnum);
-        s << "enum " << enumName << " {};" << endl;
+    } else {
+        enumName = cppEnum->name();
+        if (cppEnum->enclosingClass())
+            enumName = cppEnum->enclosingClass()->qualifiedCppName() + "::" + enumName;
     }
-#endif
 
-    s << "template<> inline PyTypeObject* SbkType<" << enumName << " >() ";
+    s << "template<> inline PyTypeObject* SbkType< ::" << enumName << " >() ";
     s << "{ return " << cpythonTypeNameExt(cppEnum->typeEntry()) << "; }\n";
 
     FlagsTypeEntry* flag = cppEnum->typeEntry()->flags();
     if (flag) {
-        s <<  "template<> inline PyTypeObject* SbkType<" << flag->name() << " >() "
+        s <<  "template<> inline PyTypeObject* SbkType< ::" << flag->name() << " >() "
           << "{ return " << cpythonTypeNameExt(flag) << "; }\n";
     }
 }
 
 void HeaderGenerator::writeSbkTypeFunction(QTextStream& s, const AbstractMetaClass* cppClass)
 {
-    s <<  "template<> inline PyTypeObject* SbkType<" << cppClass->qualifiedCppName() << " >() "
+    s <<  "template<> inline PyTypeObject* SbkType< ::" << cppClass->qualifiedCppName() << " >() "
       <<  "{ return reinterpret_cast<PyTypeObject*>(" << cpythonTypeNameExt(cppClass->typeEntry()) << "); }\n";
 }
 
-void HeaderGenerator::writeSbkCopyCppObjectFunction(QTextStream& s, const AbstractMetaClass* metaClass)
-{
-    if (!metaClass->typeEntry()->isValue() || !shouldGenerateCppWrapper(metaClass))
-        return;
-    QString className = metaClass->qualifiedCppName();
-    s << "template <>" << endl;
-    s << "struct SbkTypeInfo<" << className << " >" << endl;
-    s << '{' << endl;
-    s << INDENT << "static const bool isCppWrapper = true;" << endl;
-    s << "};" << endl;
-}
-
 void HeaderGenerator::writeTypeConverterImpl(QTextStream& s, const TypeEntry* type)
 {
     if (type->hasNativeConversionRule())
@@ -521,19 +551,21 @@ void HeaderGenerator::writeTypeConverterImpl(QTextStream& s, const TypeEntry* ty
     if (!hasImplicitConversions)
         return;
 
+    const QString typeName = "::" + type->qualifiedCppName();
+
     // Write Converter<T>::isConvertible
-    s << "inline bool Shiboken::Converter<" << type->name() << " >::isConvertible(PyObject* pyobj)" << endl;
+    s << "inline bool Shiboken::Converter< " << typeName << " >::isConvertible(PyObject* pyobj)" << endl;
     s << '{' << endl;
 
     if (type->isValue()) {
-        s << INDENT << "if (ValueTypeConverter<" << type->name() << " >::isConvertible(pyobj))" << endl;
+        s << INDENT << "if (ValueTypeConverter< " << typeName << " >::isConvertible(pyobj))" << endl;
         Indentation indent(INDENT);
         s << INDENT << "return true;" << endl;
     }
 
 
-    s << INDENT << "SbkBaseWrapperType* shiboType = reinterpret_cast<SbkBaseWrapperType*>(SbkType<";
-    s << type->name() << " >());" << endl;
+    s << INDENT << "SbkObjectType* shiboType = reinterpret_cast<SbkObjectType*>(SbkType< ";
+    s << typeName << " >());" << endl;
     s << INDENT << "return ";
     bool isFirst = true;
     foreach (const AbstractMetaFunction* ctor, implicitConvs) {
@@ -551,31 +583,25 @@ void HeaderGenerator::writeTypeConverterImpl(QTextStream& s, const TypeEntry* ty
     s << endl;
     {
         Indentation indent(INDENT);
-        s << INDENT << " || (shiboType->ext_isconvertible && shiboType->ext_isconvertible(pyobj));" << endl;
+        s << INDENT << " || (ObjectType::isExternalConvertible(shiboType, pyobj));" << endl;
     }
     s << '}' << endl << endl;
 
     // Write Converter<T>::toCpp function
-    s << "inline " << type->name() << " Shiboken::Converter<" << type->name() << " >::toCpp(PyObject* pyobj)" << endl;
+    s << "inline " << typeName << " Shiboken::Converter< " << typeName << " >::toCpp(PyObject* pyobj)" << endl;
     s << '{' << endl;
 
-    s << INDENT << "if (!Shiboken_TypeCheck(pyobj, " << type->name() << ")) {" << endl;
+    s << INDENT << "if (PyObject_TypeCheck(pyobj, SbkType< " << typeName << " >()))" << endl;
     {
         Indentation indent(INDENT);
-        s << INDENT << "SbkBaseWrapperType* shiboType = reinterpret_cast<SbkBaseWrapperType*>(SbkType<";
-        s << type->name() << " >());" << endl;
+        s << INDENT << "return *" << cpythonWrapperCPtr(type, "pyobj") << ';' << endl;
     }
-    bool firstImplicitIf = true;
+
     foreach (const AbstractMetaFunction* ctor, implicitConvs) {
         if (ctor->isModifiedRemoved())
             continue;
 
-        Indentation indent(INDENT);
-        s << INDENT;
-        if (firstImplicitIf)
-            firstImplicitIf = false;
-        else
-            s << "else ";
+        s << INDENT << "else ";
 
         QString typeCheck;
         QString toCppConv;
@@ -598,20 +624,36 @@ void HeaderGenerator::writeTypeConverterImpl(QTextStream& s, const TypeEntry* ty
     }
 
     {
-        Indentation indent(INDENT);
-        s << INDENT << "else if (shiboType->ext_isconvertible && shiboType->ext_tocpp && shiboType->ext_isconvertible(pyobj)) {" << endl;
+        s << INDENT << "else" << endl;
         {
             Indentation indent(INDENT);
-            s << INDENT << type->name() << "* cptr = reinterpret_cast<" << type->name() << "*>(shiboType->ext_tocpp(pyobj));" << endl;
-            s << INDENT << "std::auto_ptr<" << type->name() << " > cptr_auto_ptr(cptr);" << endl;
-            s << INDENT << "return *cptr;" << endl;
+            s << INDENT << "return Shiboken::ValueTypeConverter< " << typeName << " >::toCpp(pyobj);" << endl;
         }
-        s << INDENT << '}' << endl;
     }
-
-    s << INDENT << '}' << endl;
-
-    s << INDENT << "return *" << cpythonWrapperCPtr(type, "pyobj") << ';' << endl;
     s << '}' << endl << endl;
 }
 
+void HeaderGenerator::writeInheritedOverloads(QTextStream& s)
+{
+    foreach (const AbstractMetaFunction* func, m_inheritedOverloads) {
+        s << INDENT << "inline ";
+        s << functionSignature(func, "", "", Generator::EnumAsInts|Generator::OriginalTypeDescription) << " { ";
+        s << (func->type() ? "return " : "");
+        s << func->ownerClass()->qualifiedCppName() << "::" << func->originalName() << '(';
+        QStringList args;
+        foreach (const AbstractMetaArgument* arg, func->arguments()) {
+            QString argName = arg->name();
+            const TypeEntry* enumTypeEntry = 0;
+            if (arg->type()->isFlags())
+                enumTypeEntry = reinterpret_cast<const FlagsTypeEntry*>(arg->type()->typeEntry())->originator();
+            else if (arg->type()->isEnum())
+                enumTypeEntry = arg->type()->typeEntry();
+            if (enumTypeEntry)
+                argName = QString("%1(%2)").arg(arg->type()->cppSignature()).arg(argName);
+            args << argName;
+        }
+        s << args.join(", ") << ')';
+        s << "; }" << endl;
+    }
+}
+
