@@ -382,17 +382,18 @@ static QString searchForEnumScope(const AbstractMetaClass* metaClass, const QStr
  */
 QString ShibokenGenerator::guessScopeForDefaultValue(const AbstractMetaFunction* func, const AbstractMetaArgument* arg)
 {
-    if (arg->defaultValueExpression().isEmpty())
+    QString value = getDefaultValue(func, arg);
+    if (value.isEmpty())
         return QString();
 
     static QRegExp enumValueRegEx("^([A-Za-z_]\\w*)?$");
-    QString value = arg->defaultValueExpression();
     QString prefix;
     QString suffix;
 
     if (arg->type()->isEnum()) {
         const AbstractMetaEnum* metaEnum = findAbstractMetaEnum(arg->type());
-        prefix = resolveScopePrefix(metaEnum->enclosingClass(), value);
+        if (metaEnum)
+            prefix = resolveScopePrefix(metaEnum->enclosingClass(), value);
     } else if (arg->type()->isFlags()) {
         static QRegExp numberRegEx("^\\d+$"); // Numbers to flags
         if (numberRegEx.exactMatch(value)) {
@@ -485,8 +486,8 @@ QString ShibokenGenerator::cpythonWrapperCPtr(const AbstractMetaType* metaType,
 
 QString ShibokenGenerator::cpythonWrapperCPtr(const TypeEntry* type, QString argName)
 {
-    if (type->isValue() || type->isObject())
-        return baseConversionString(type->qualifiedCppName() + '*') + "toCpp(" + argName + ')';
+    if (ShibokenGenerator::isWrapperType(type))
+        return baseConversionString( "::" + type->qualifiedCppName() + '*') + "toCpp(" + argName + ')';
     return QString();
 }
 
@@ -518,9 +519,7 @@ void ShibokenGenerator::writeBaseConversion(QTextStream& s, const TypeEntry* typ
             typeName = protectedEnumSurrogateName(metaEnum);
     } else {
         typeName = type->qualifiedCppName().trimmed();
-        if (!type->isCppPrimitive())
-            typeName.prepend("::");
-        if (type->isObject())
+        if (ShibokenGenerator::isObjectType(type))
             typeName.append('*');
     }
     s << baseConversionString(typeName);
@@ -541,8 +540,6 @@ void ShibokenGenerator::writeBaseConversion(QTextStream& s, const AbstractMetaTy
         if (type->isContainer() || type->isFlags() || type->isEnum() || (type->isConstant() && type->isReference()))
             options |= Generator::ExcludeReference;
         typeName = translateTypeForWrapperMethod(type, context, options).trimmed();
-        if (!type->typeEntry()->isCppPrimitive())
-            typeName.prepend("::");
     }
 
     s << baseConversionString(typeName);
@@ -636,7 +633,7 @@ QString ShibokenGenerator::cpythonBaseName(const AbstractMetaClass* metaClass)
 QString ShibokenGenerator::cpythonBaseName(const TypeEntry* type)
 {
     QString baseName;
-    if ((type->isObject() || type->isValue() || type->isNamespace())) { // && !type->isReference()) {
+    if (ShibokenGenerator::isWrapperType(type) || type->isNamespace()) { // && !type->isReference()) {
         baseName = "Sbk_" + type->name();
     } else if (type->isPrimitive()) {
         const PrimitiveTypeEntry* ptype = (const PrimitiveTypeEntry*) type;
@@ -798,6 +795,51 @@ bool ShibokenGenerator::isPairContainer(const AbstractMetaType* type)
             && ((ContainerTypeEntry*)type->typeEntry())->type() == ContainerTypeEntry::PairContainer;
 }
 
+bool ShibokenGenerator::isObjectType(const TypeEntry* type)
+{
+    if (type->isComplex())
+        return ShibokenGenerator::isObjectType((const ComplexTypeEntry*)type);
+    return type->isObject();
+}
+bool ShibokenGenerator::isObjectType(const ComplexTypeEntry* type)
+{
+    return type->isObject() || type->isQObject();
+}
+bool ShibokenGenerator::isObjectType(const AbstractMetaClass* metaClass)
+{
+    return ShibokenGenerator::isObjectType(metaClass->typeEntry());
+}
+bool ShibokenGenerator::isObjectType(const AbstractMetaType* metaType)
+{
+    return metaType->isObject() || metaType->isQObject();
+}
+
+bool ShibokenGenerator::isWrapperType(const TypeEntry* type)
+{
+    if (type->isComplex())
+        return ShibokenGenerator::isWrapperType((const ComplexTypeEntry*)type);
+    return type->isObject() || type->isValue();
+}
+bool ShibokenGenerator::isWrapperType(const ComplexTypeEntry* type)
+{
+    return ShibokenGenerator::isObjectType(type) || type->isValue();
+}
+bool ShibokenGenerator::isWrapperType(const AbstractMetaType* metaType)
+{
+    return ShibokenGenerator::isObjectType(metaType)
+            || metaType->typeEntry()->isValue();
+}
+
+bool ShibokenGenerator::isPointer(const AbstractMetaType* type)
+{
+    return type->indirections() > 0;
+}
+
+bool ShibokenGenerator::isPointerToWrapperType(const AbstractMetaType* type)
+{
+    return ShibokenGenerator::isObjectType(type) || type->isValuePointer();
+}
+
 bool ShibokenGenerator::shouldDereferenceArgumentPointer(const AbstractMetaArgument* arg)
 {
     return shouldDereferenceAbstractMetaTypePointer(arg->type());
@@ -805,10 +847,9 @@ bool ShibokenGenerator::shouldDereferenceArgumentPointer(const AbstractMetaArgum
 
 bool ShibokenGenerator::shouldDereferenceAbstractMetaTypePointer(const AbstractMetaType* metaType)
 {
-    const TypeEntry* type = metaType->typeEntry();
-    bool isPointer = metaType->indirections() > 0;
-    bool isCppClass = type->isValue() || type->isObject();
-    return isCppClass && !isPointer && (metaType->isValue() || metaType->isReference());
+    return ShibokenGenerator::isWrapperType(metaType)
+            && !ShibokenGenerator::isPointer(metaType)
+            && (metaType->isValue() || metaType->isReference());
 }
 
 bool ShibokenGenerator::visibilityModifiedToPrivate(const AbstractMetaFunction* func)
@@ -864,6 +905,8 @@ QString ShibokenGenerator::guessCPythonCheckFunction(const QString& type)
         metaType = 0;
     } else if (type == "PyTypeObject") {
         retval = "PyType_Check";
+    } else if (type == "PyBuffer") {
+        retval = "Shiboken::Buffer::checkType";
     } else {
         retval = QString("%1_Check").arg(type);
     }
@@ -1103,6 +1146,34 @@ ShibokenGenerator::ExtendedConverterData ShibokenGenerator::getExtendedConverter
     return extConvs;
 }
 
+static QString getArgumentsFromMethodCall(const QString& str)
+{
+    // It would be way nicer to be able to use a Perl like
+    // regular expression that accepts temporary variables
+    // to count the parenthesis.
+    // For more information check this:
+    // http://perl.plover.com/yak/regex/samples/slide083.html
+    static QString funcCall("%CPPSELF.%FUNCTION_NAME");
+    int pos = str.indexOf(funcCall);
+    if (pos == -1)
+        return QString();
+    pos = pos + funcCall.count();
+    while (str.at(pos) == ' ' || str.at(pos) == '\t')
+        ++pos;
+    if (str.at(pos) == '(')
+        ++pos;
+    int begin = pos;
+    int counter = 1;
+    while (counter != 0) {
+        if (str.at(pos) == '(')
+            ++counter;
+        else if (str.at(pos) == ')')
+            --counter;
+        ++pos;
+    }
+    return str.mid(begin, pos-begin-1);
+}
+
 void ShibokenGenerator::writeCodeSnips(QTextStream& s,
                                        const CodeSnipList& codeSnips,
                                        CodeSnip::Position position,
@@ -1113,6 +1184,8 @@ void ShibokenGenerator::writeCodeSnips(QTextStream& s,
 {
     static QRegExp toPythonRegex("%CONVERTTOPYTHON\\[([^\\[]*)\\]");
     static QRegExp toCppRegex("%CONVERTTOCPP\\[([^\\[]*)\\]");
+    static QRegExp isConvertibleRegex("%ISCONVERTIBLE\\[([^\\[]*)\\]");
+    static QRegExp checkTypeRegex("%CHECKTYPE\\[([^\\[]*)\\]");
     static QRegExp pyArgsRegex("%PYARG_(\\d+)");
 
     // detect is we should use pyargs instead of args as variable name for python arguments
@@ -1143,12 +1216,18 @@ void ShibokenGenerator::writeCodeSnips(QTextStream& s,
             code.replace("%CPPTYPE", context->name());
         }
 
-        // replace "toPython "converters
+        // replace "toPython" converters
         code.replace(toPythonRegex, "Shiboken::Converter<\\1 >::toPython");
 
-        // replace "toCpp "converters
+        // replace "toCpp" converters
         code.replace(toCppRegex, "Shiboken::Converter<\\1 >::toCpp");
 
+        // replace "isConvertible" check
+        code.replace(isConvertibleRegex, "Shiboken::Converter<\\1 >::isConvertible");
+
+        // replace "checkType" check
+        code.replace(checkTypeRegex, "Shiboken::Converter<\\1 >::checkType");
+
         if (func) {
             // replace %PYARG_# variables
             code.replace("%PYARG_0", PYTHON_RETURN_VAR);
@@ -1190,8 +1269,8 @@ void ShibokenGenerator::writeCodeSnips(QTextStream& s,
                 code.replace("%0.", QString("%1->").arg("cptr"));
                 code.replace("%0", "cptr");
             } else if (func->type()) {
-                QString returnValueOp = func->type()->isObject() || func->type()->isQObject() ? "%1->" : "%1.";
-                if (func->type()->typeEntry()->isValue() || func->type()->typeEntry()->isObject())
+                QString returnValueOp = ShibokenGenerator::isPointerToWrapperType(func->type()) ? "%1->" : "%1.";
+                if (ShibokenGenerator::isWrapperType(func->type()))
                     code.replace("%0.", returnValueOp.arg(CPP_RETURN_VAR));
                 code.replace("%0", CPP_RETURN_VAR);
             }
@@ -1221,6 +1300,25 @@ void ShibokenGenerator::writeCodeSnips(QTextStream& s,
                 if (func->isComparisonOperator())
                     replacement = "%1.";
 
+                if (func->isVirtual() && !func->isAbstract() && (!avoidProtectedHack() || !func->isProtected())) {
+                    QString methodCallArgs = getArgumentsFromMethodCall(code);
+                    if (!methodCallArgs.isNull()) {
+                        if (func->name() == "metaObject") {
+                            QString wrapperClassName = wrapperName(func->ownerClass());
+                            QString cppSelfVar = avoidProtectedHack() ? QString("%CPPSELF") : QString("reinterpret_cast<%1*>(%CPPSELF)").arg(wrapperClassName);
+                            code.replace(QString("%CPPSELF.%FUNCTION_NAME(%1)").arg(methodCallArgs),
+                                         QString("(Shiboken::Object::hasCppWrapper(reinterpret_cast<SbkObject*>(%1))"
+                                                 " ? %2->::%3::%FUNCTION_NAME(%4)"
+                                                 " : %CPPSELF.%FUNCTION_NAME(%4))").arg(pySelf).arg(cppSelfVar).arg(wrapperClassName).arg(methodCallArgs));
+                        } else {
+                            code.replace(QString("%CPPSELF.%FUNCTION_NAME(%1)").arg(methodCallArgs),
+                                         QString("(Shiboken::Object::hasCppWrapper(reinterpret_cast<SbkObject*>(%1))"
+                                                 " ? %CPPSELF->::%TYPE::%FUNCTION_NAME(%2)"
+                                                 " : %CPPSELF.%FUNCTION_NAME(%2))").arg(pySelf).arg(methodCallArgs));
+                        }
+                    }
+                }
+
                 code.replace("%CPPSELF.", replacement.arg(cppSelf));
                 code.replace("%CPPSELF", cppSelf);
 
@@ -1467,10 +1565,12 @@ AbstractMetaFunctionList ShibokenGenerator::getMethodsWithBothStaticAndNonStatic
 AbstractMetaClassList ShibokenGenerator::getBaseClasses(const AbstractMetaClass* metaClass) const
 {
     AbstractMetaClassList baseClasses;
-    foreach (QString parent, metaClass->baseClassNames()) {
-        AbstractMetaClass* clazz = classes().findClass(parent);
-        if (clazz)
-            baseClasses << clazz;
+    if (metaClass) {
+        foreach (QString parent, metaClass->baseClassNames()) {
+            AbstractMetaClass* clazz = classes().findClass(parent);
+            if (clazz)
+                baseClasses << clazz;
+        }
     }
     return baseClasses;
 }
@@ -1487,10 +1587,12 @@ const AbstractMetaClass* ShibokenGenerator::getMultipleInheritingClass(const Abs
 AbstractMetaClassList ShibokenGenerator::getAllAncestors(const AbstractMetaClass* metaClass) const
 {
     AbstractMetaClassList result;
-    AbstractMetaClassList baseClasses = getBaseClasses(metaClass);
-    foreach (AbstractMetaClass* base, baseClasses) {
-        result.append(base);
-        result.append(getAllAncestors(base));
+    if (metaClass) {
+        AbstractMetaClassList baseClasses = getBaseClasses(metaClass);
+        foreach (AbstractMetaClass* base, baseClasses) {
+            result.append(base);
+            result.append(getAllAncestors(base));
+        }
     }
     return result;
 }
@@ -1515,7 +1617,7 @@ QString ShibokenGenerator::extendedToCppFunctionName(const TypeEntry* targetType
 bool ShibokenGenerator::isCopyable(const AbstractMetaClass *metaClass)
 
 {
-    if (metaClass->isNamespace() || metaClass->typeEntry()->isObject())
+    if (metaClass->isNamespace() || ShibokenGenerator::isObjectType(metaClass))
         return false;
     else if (metaClass->typeEntry()->copyable() == ComplexTypeEntry::Unknown)
         return metaClass->hasCloneOperator();
@@ -1732,3 +1834,19 @@ Generator::Options ShibokenGenerator::getConverterOptions(const AbstractMetaType
     return flags;
 }
 
+QString  ShibokenGenerator::getDefaultValue(const AbstractMetaFunction* func, const AbstractMetaArgument* arg)
+{
+    if (!arg->defaultValueExpression().isEmpty())
+        return arg->defaultValueExpression();
+
+    //Check modifications
+    foreach(FunctionModification  m, func->modifications()) {
+        foreach(ArgumentModification am, m.argument_mods) {
+            if (am.index == (arg->argumentIndex() + 1))
+                return am.replacedDefaultExpression;
+        }
+    }
+    return QString();
+}
+
+
