@@ -183,13 +183,16 @@ void SbkDeallocWrapper(PyObject* pyObj)
             Shiboken::DtorCallerVisitor visitor(sbkObj);
             Shiboken::walkThroughClassHierarchy(pyObj->ob_type, &visitor);
         } else {
+            void* cptr = sbkObj->d->cptr[0];
+            Shiboken::Object::deallocData(sbkObj, true);
+
             Shiboken::ThreadStateSaver threadSaver;
             threadSaver.save();
-            sbkType->d->cpp_dtor(sbkObj->d->cptr[0]);
+            sbkType->d->cpp_dtor(cptr);
         }
+    } else {
+        Shiboken::Object::deallocData(sbkObj, true);
     }
-    //Always destroy object data during the python object destruction
-    Shiboken::Object::deallocData(sbkObj, true);
 }
 
 void SbkDeallocWrapperWithPrivateDtor(PyObject* self)
@@ -207,14 +210,16 @@ void SbkObjectTypeDealloc(PyObject* pyObj)
 
     PyObject_GC_UnTrack(pyObj);
     Py_TRASHCAN_SAFE_BEGIN(pyObj);
-    if(sbkType->d->user_data && sbkType->d->d_func) {
-        sbkType->d->d_func(sbkType->d->user_data);
-        sbkType->d->user_data = 0;
+    if (sbkType->d) {
+        if(sbkType->d->user_data && sbkType->d->d_func) {
+            sbkType->d->d_func(sbkType->d->user_data);
+            sbkType->d->user_data = 0;
+        }
+        free(sbkType->d->original_name);
+        sbkType->d->original_name = 0;
+        delete sbkType->d;
+        sbkType->d = 0;
     }
-    free(sbkType->d->original_name);
-    sbkType->d->original_name = 0;
-    delete sbkType->d;
-    sbkType->d = 0;
     Py_TRASHCAN_SAFE_END(pyObj);
 }
 
@@ -245,7 +250,6 @@ PyObject* SbkObjectTypeTpNew(PyTypeObject* metatype, PyObject* args, PyObject* k
 
     // The meta type creates a new type when the Python programmer extends a wrapped C++ class.
     SbkObjectType* newType = reinterpret_cast<SbkObjectType*>(PyType_Type.tp_new(metatype, args, kwds));
-
     if (!newType)
         return 0;
 
@@ -319,9 +323,9 @@ PyObject* SbkObjectTpNew(PyTypeObject* subtype, PyObject*, PyObject*)
 namespace Shiboken
 {
 
-static void decRefPyObjectList(const std::list<SbkObject*> &pyObj);
+static void decRefPyObjectList(const std::list<PyObject*> &pyObj, PyObject* skip = 0);
 
-void walkThroughClassHierarchy(PyTypeObject* currentType, HierarchyVisitor* visitor)
+static void _walkThroughClassHierarchy(PyTypeObject* currentType, HierarchyVisitor* visitor)
 {
     PyObject* bases = currentType->tp_bases;
     Py_ssize_t numBases = PyTuple_GET_SIZE(bases);
@@ -333,23 +337,33 @@ void walkThroughClassHierarchy(PyTypeObject* currentType, HierarchyVisitor* visi
         } else {
             SbkObjectType* sbkType = reinterpret_cast<SbkObjectType*>(type);
             if (sbkType->d->is_user_type)
-                walkThroughClassHierarchy(type, visitor);
+                _walkThroughClassHierarchy(type, visitor);
             else
                 visitor->visit(sbkType);
         }
         if (visitor->wasFinished())
-            return;
+            break;
     }
 }
 
+void walkThroughClassHierarchy(PyTypeObject* currentType, HierarchyVisitor* visitor)
+{
+    _walkThroughClassHierarchy(currentType, visitor);
+    visitor->done();
+}
+
+
 bool importModule(const char* moduleName, PyTypeObject*** cppApiPtr)
 {
     PyObject* sysModules = PyImport_GetModuleDict();
     PyObject* module = PyDict_GetItemString(sysModules, moduleName);
-    if (!module)
+    if (!module) {
         module = PyImport_ImportModule(moduleName);
-    else
+        if (!module)
+            return false;
+    } else {
         Py_INCREF(module);
+    }
 
     Shiboken::AutoDecRef cppApi(PyObject_GetAttrString(module, "_Cpp_Api"));
     Py_DECREF(module);
@@ -367,10 +381,19 @@ bool importModule(const char* moduleName, PyTypeObject*** cppApiPtr)
 
 void DtorCallerVisitor::visit(SbkObjectType* node)
 {
-    Shiboken::ThreadStateSaver threadSaver;
-    threadSaver.save();
-    node->d->cpp_dtor(m_pyObj->d->cptr[m_count]);
-    m_count++;
+    m_ptrs.push_back(std::make_pair(m_pyObj->d->cptr[m_ptrs.size()], node));
+}
+
+void DtorCallerVisitor::done()
+{
+    Shiboken::Object::deallocData(m_pyObj, true);
+
+    std::list<std::pair<void*, SbkObjectType*> >::const_iterator it = m_ptrs.begin();
+    for (; it != m_ptrs.end(); ++it) {
+        Shiboken::ThreadStateSaver threadSaver;
+        threadSaver.save();
+        it->second->d->cpp_dtor(it->first);
+    }
 }
 
 void init()
@@ -474,11 +497,12 @@ std::list<SbkObject*> splitPyObject(PyObject* pyObj)
     return result;
 }
 
-static void decRefPyObjectList(const std::list<SbkObject*>& lst)
+static void decRefPyObjectList(const std::list<PyObject*>& lst, PyObject *skip)
 {
-    std::list<SbkObject*>::const_iterator iter = lst.begin();
+    std::list<PyObject*>::const_iterator iter = lst.begin();
     while(iter != lst.end()) {
-        Py_DECREF(*iter);
+        if (*iter != skip)
+            Py_DECREF(*iter);
         ++iter;
     }
 }
@@ -532,19 +556,19 @@ bool isExternalConvertible(SbkObjectType* self, PyObject* obj)
     return self->d->ext_isconvertible && self->d->ext_isconvertible(obj);
 }
 
-bool hasCast(SbkObjectType* self)
+bool hasCast(SbkObjectType* type)
 {
-    return self->d->mi_specialcast;
+    return type->d->mi_specialcast;
 }
 
-void* cast(SbkObjectType* self, SbkObject* obj, PyTypeObject *target)
+void* cast(SbkObjectType* sourceType, SbkObject* obj, PyTypeObject* targetType)
 {
-    return self->d->mi_specialcast(Object::cppPointer(obj, target), reinterpret_cast<SbkObjectType*>(target));
+    return sourceType->d->mi_specialcast(Object::cppPointer(obj, targetType), reinterpret_cast<SbkObjectType*>(targetType));
 }
 
-void setCastFunction(SbkObjectType* self, SpecialCastFunction func)
+void setCastFunction(SbkObjectType* type, SpecialCastFunction func)
 {
-    self->d->mi_specialcast = func;
+    type->d->mi_specialcast = func;
 }
 
 void setOriginalName(SbkObjectType* self, const char* name)
@@ -793,7 +817,9 @@ void* cppPointer(SbkObject* pyObj, PyTypeObject* desiredType)
     int idx = 0;
     if (reinterpret_cast<SbkObjectType*>(type)->d->is_multicpp)
         idx = getTypeIndexOnHierarchy(type, desiredType);
-    return pyObj->d->cptr[idx];
+    if (pyObj->d->cptr)
+        return pyObj->d->cptr[idx];
+    return 0;
 }
 
 bool setCppPointer(SbkObject* sbkObj, PyTypeObject* desiredType, void* cptr)
@@ -929,6 +955,11 @@ void destroy(SbkObject* self, void* cppData)
     if (cppData && Shiboken::BindingManager::instance().hasWrapper(cppData)) {
         // Remove from BindingManager
         Shiboken::BindingManager::instance().releaseWrapper(self);
+        self->d->hasOwnership = false;
+
+        // the cpp object instance was deleted
+        delete[] self->d->cptr;
+        self->d->cptr = 0;
     }
 
     // After this point the object can be death do not use the self pointer bellow
@@ -938,6 +969,11 @@ void removeParent(SbkObject* child, bool giveOwnershipBack, bool keepReference)
 {
     ParentInfo* pInfo = child->d->parentInfo;
     if (!pInfo || !pInfo->parent) {
+        if (pInfo && pInfo->hasWrapperRef) {
+            pInfo->hasWrapperRef = false;
+            delete pInfo;
+            child->d->parentInfo = 0;
+        }
         return;
     }
     ChildrenList& oldBrothers = pInfo->parent->d->parentInfo->children;
@@ -951,8 +987,15 @@ void removeParent(SbkObject* child, bool giveOwnershipBack, bool keepReference)
     pInfo->parent = 0;
 
     // This will keep the wrapper reference, will wait for wrapper destruction to remove that
-    if (keepReference && child->d->containsCppWrapper)
+    if (keepReference &&
+        child->d->containsCppWrapper) {
+        //If have already a extra ref remove this one
+        if (pInfo->hasWrapperRef)
+            Py_CLEAR(child);
+        else
+            pInfo->hasWrapperRef = true;
         return;
+    }
 
     // Transfer ownership back to Python
     child->d->hasOwnership = giveOwnershipBack;
@@ -1042,13 +1085,14 @@ void deallocData(SbkObject* self, bool cleanup)
         clearReferences(self);
     }
 
-    // Remove from BindingManager
-    Shiboken::BindingManager::instance().releaseWrapper(self);
-
+    if (self->d->cptr) {
+        // Remove from BindingManager
+        Shiboken::BindingManager::instance().releaseWrapper(self);
+        delete[] self->d->cptr;
+        self->d->cptr = 0;
+        delete self->d;
+    }
     Py_XDECREF(self->ob_dict);
-    delete[] self->d->cptr;
-    self->d->cptr = 0;
-    delete self->d;
     Py_TYPE(self)->tp_free(self);
 }
 
@@ -1075,26 +1119,51 @@ void keepReference(SbkObject* self, const char* key, PyObject* referredObject, b
         self->d->referredObjects = new Shiboken::RefCountMap;
 
     RefCountMap& refCountMap = *(self->d->referredObjects);
-    if (!isNone)
-        Py_INCREF(referredObject);
-
     RefCountMap::iterator iter = refCountMap.find(key);
-    if (!append && (iter != refCountMap.end())) {
-        decRefPyObjectList(iter->second);
-        refCountMap.erase(iter);
+    std::list<PyObject*> objects;
+    if (iter != refCountMap.end()) {
+        objects = (*iter).second;
+        std::list<PyObject*>::const_iterator found = std::find(objects.begin(), objects.end(), referredObject);
+
+        // skip if objects already exists
+        if (found != objects.end())
+            return;
     }
 
-    if (!isNone) {
-        if (append && (iter != refCountMap.end())) {
-            refCountMap[key].push_back(reinterpret_cast<SbkObject*>(referredObject));
+    if (append && !isNone) {
+        refCountMap[key].push_back(referredObject);
+        Py_INCREF(referredObject);
+    } else if (!append) {
+        if (objects.size() > 0)
+            decRefPyObjectList(objects, isNone ? 0 : referredObject);
+        if (isNone) {
+            if (iter != refCountMap.end())
+                refCountMap.erase(iter);
         } else {
-            std::list<SbkObject*> new_list;
-            new_list.push_back(reinterpret_cast<SbkObject*>(referredObject));
-            refCountMap[key] = new_list;;
+            objects.clear();
+            objects.push_back(referredObject);
+            refCountMap[key] = objects;
+            Py_INCREF(referredObject);
         }
     }
 }
 
+void removeReference(SbkObject* self, const char* key, PyObject* referredObject)
+{
+    if (!referredObject || (referredObject == Py_None))
+        return;
+
+    if (!self->d->referredObjects)
+        return;
+
+    RefCountMap& refCountMap = *(self->d->referredObjects);
+    RefCountMap::iterator iter = refCountMap.find(key);
+    if (iter != refCountMap.end()) {
+        decRefPyObjectList(iter->second);
+        refCountMap.erase(iter);
+    }
+}
+
 void clearReferences(SbkObject* self)
 {
     if (!self->d->referredObjects)
