@@ -5,31 +5,19 @@
  *
  * Contact: PySide team <contact@pyside.org>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * version 2.1 as published by the Free Software Foundation. Please
- * review the following information to ensure the GNU Lesser General
- * Public License version 2.1 requirements will be met:
- * http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * As a special exception to the GNU Lesser General Public License
- * version 2.1, the object code form of a "work that uses the Library"
- * may incorporate material from a header file that is part of the
- * Library.  You may distribute such object code under terms of your
- * choice, provided that the incorporated material (i) does not exceed
- * more than 5% of the total size of the Library; and (ii) is limited to
- * numerical parameters, data structure layouts, accessors, macros,
- * inline functions and templates.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef SAMPLENAMESPACE_H
@@ -41,6 +29,24 @@
 
 class ObjectType;
 
+// Anonymous global enum
+enum {
+    AnonymousGlobalEnum_Value0,
+    AnonymousGlobalEnum_Value1
+};
+
+// Invisible namespace
+namespace Invisible
+{
+
+enum EnumOnNamespace {
+    Option1     = 1,
+    Option2     = 2,
+    Option3     = 3
+};
+
+};
+
 namespace SampleNamespace
 {
 
@@ -62,11 +68,19 @@ enum OutValue {
     TwoOut
 };
 
+// Anonymous non-global enum.
+// This counts as a class enum, since C++ namespaces
+// are represented as classes in Python.
+enum {
+    AnonymousClassEnum_Value0,
+    AnonymousClassEnum_Value1
+};
+
 LIBSAMPLE_API OutValue enumInEnumOut(InValue in);
 
 LIBSAMPLE_API int getNumber(Option opt);
 
-LIBSAMPLE_API inline double powerOfTwo(double num) {
+inline double powerOfTwo(double num) {
     return num * num;
 }
 
@@ -88,7 +102,7 @@ public:
                 NiceValue1, NiceValue2
             };
 
-            int someMethod(SomeInnerClass*) { return 0; }
+            inline int someMethod(SomeInnerClass*) { return 0; }
             virtual OkThisIsRecursiveEnough* someVirtualMethod(OkThisIsRecursiveEnough* arg) { return arg; }
         };
     };
@@ -97,8 +111,12 @@ public:
 class DerivedFromNamespace : public SomeClass::SomeInnerClass::OkThisIsRecursiveEnough
 {
 public:
+    // FIXME Uncomment this when the fix for MSVC is available
+    // only to cause namespace confusion
+//    enum SampleNamespace {
+//    };
     virtual OkThisIsRecursiveEnough* someVirtualMethod(OkThisIsRecursiveEnough* arg) { return arg; }
-    OkThisIsRecursiveEnough* methodReturningTypeFromParentScope() { return 0; }
+    inline OkThisIsRecursiveEnough* methodReturningTypeFromParentScope() { return 0; }
 };
 
 // The combination of the following two overloaded methods could trigger a
