@@ -15,6 +15,7 @@
 #include "list.h"
 #include "listuser.h"
 #include "mapuser.h"
+#include "modelindex.h"
 #include "modifications.h"
 #include "modified_constructor.h"
 #include "multiple_derived.h"
@@ -40,6 +41,7 @@
 #include "rect.h"
 #include "reference.h"
 #include "removednamespaces.h"
+#include "sample.h"
 #include "samplenamespace.h"
 #include "simplefile.h"
 #include "size.h"
